//
//  ImageViewController.swift
//  Cassini
//
//  Created by CS193p Instructor.
//  Copyright © 2017 CS193p Instructor. All rights reserved.
//

import UIKit

class ImageViewController: UIViewController, UIScrollViewDelegate
{
    var imageURL: URL? {
        didSet {
            image = nil
            if view.window != nil {
                fetchImage()
            }
        }
    }
    private var autoZoomed = true
    private var image: UIImage? {
        get {
            return imageView.image
        }
        set {
            imageView.image = newValue
            imageView.sizeToFit()
            scrollView?.contentSize = imageView.frame.size
            spinner?.stopAnimating()
            autoZoomed = true
            zoomScaleToFit()
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        if imageView.image == nil {
            fetchImage()
        }
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        zoomScaleToFit()
    }

    @IBOutlet weak var spinner: UIActivityIndicatorView!
    @IBOutlet weak var scrollView: UIScrollView! {
        didSet {
            scrollView.minimumZoomScale = 0.03
            scrollView.maximumZoomScale = 5.0
            scrollView.delegate = self
            scrollView.addSubview(imageView)
        }
    }
    
    func viewForZooming(in scrollView: UIScrollView) -> UIView? {
        return imageView
    }
    
    // Убираем автоматическую "подгонку" после того, как пользователь выполняет zoom
    // с помощью жеста pinching
    func scrollViewWillBeginZooming(_ scrollView: UIScrollView, with view: UIView?) {
        autoZoomed = false
    }
    
    var imageView  = UIImageView()
    
    private func fetchImage() {
         autoZoomed = true
        if let url = imageURL {
            spinner.startAnimating()
            let request = URLRequest(url: url)
            DispatchQueue.global(qos: .userInitiated).async { [weak self] in
                if let data = cache.cachedResponse(for: request)?.data,
                    let image = UIImage(data: data) {
                    DispatchQueue.main.async {
                        //    self.imageGallery?.image =  image
                        self?.image = image
                        print ("кэш")
                        self?.spinner?.stopAnimating()
                    }
                } else {
                let urlContents = try? Data(contentsOf: url)
                DispatchQueue.main.async {
                    if let imageData = urlContents, url == self?.imageURL {
                        self?.image = UIImage(data: imageData)
                    }
                }
            }
        }
    }
    }
    
    // Рассчитывает zoom масштаб для "подгонки" изображения к размеру экрана,
    // но без пустых (белых) зазоров
    private func zoomScaleToFit()
    {
        // если мы все еще в режиме "подгонки" (autoZoomed), то устанавливаем
        // свойство zoomScale только при условии, что геометрия полностью загружена
        if !autoZoomed {
            return
        }
        if let sv = scrollView , image != nil && (imageView.bounds.size.width > 0)
            && (scrollView.bounds.size.width > 0){
            
            let widthRatio = scrollView.bounds.size.width  / imageView.bounds.size.width
            let heightRatio = scrollView.bounds.size.height / self.imageView.bounds.size.height
            sv.zoomScale = (widthRatio > heightRatio) ? widthRatio : heightRatio
            sv.contentOffset = CGPoint(x: (imageView.frame.size.width - sv.frame.size.width) / 2,
                                       y: (imageView.frame.size.height - sv.frame.size.height) / 2)
        }
    }
}
